@@ -280,15 +280,15 @@ $input_dbpass = new html_inputfield(['name' => '_dbpass', 'size' => 20, 'id' =>
 
 $dsnw = rcube_db::parse_dsn($RCI->getprop('db_dsnw'));
 
-echo $select_dbtype->show($RCI->is_post ? $_POST['_dbtype'] : $dsnw['phptype']);
+echo $select_dbtype->show($RCI->is_post ? $_POST['_dbtype'] : ($dsnw['phptype'] ?? ''));
 echo '<label for="cfgdbtype">Database type</label><br />';
-echo $input_dbhost->show($RCI->is_post ? $_POST['_dbhost'] : $dsnw['hostspec']);
+echo $input_dbhost->show($RCI->is_post ? $_POST['_dbhost'] : ($dsnw['hostspec'] ?? ''));
 echo '<label for="cfgdbhost">Database server (omit for sqlite)</label><br />';
-echo $input_dbname->show($RCI->is_post ? $_POST['_dbname'] : $dsnw['database']);
+echo $input_dbname->show($RCI->is_post ? $_POST['_dbname'] : ($dsnw['database'] ?? ''));
 echo '<label for="cfgdbname">Database name (use absolute path and filename for sqlite)</label><br />';
-echo $input_dbuser->show($RCI->is_post ? $_POST['_dbuser'] : $dsnw['username']);
-echo '<label for="cfgdbuser">Database user name (needs write permissions)(omit for sqlite)</label><br />';
-echo $input_dbpass->show($RCI->is_post ? $_POST['_dbpass'] : $dsnw['password']);
+echo $input_dbuser->show($RCI->is_post ? $_POST['_dbuser'] : ($dsnw['username'] ?? ''));
+echo '<label for="cfgdbuser">Database user name (needs write permissions) (omit for sqlite)</label><br />';
+echo $input_dbpass->show($RCI->is_post ? $_POST['_dbpass'] : ($dsnw['password'] ?? ''));
 echo '<label for="cfgdbpass">Database password (omit for sqlite)</label><br />';
 
 ?>
