@@ -274,7 +274,7 @@ class rcube
     /**
      * Create SMTP object and connect to server
      *
-     * @param boolean $connect True if connection should be established
+     * @param bool $connect True if connection should be established
      */
     public function smtp_init($connect = false)
     {
@@ -958,7 +958,7 @@ class rcube
         $iv_size = openssl_cipher_iv_length($method);
         $tag     = null;
 
-        if (preg_match('/^##(.{16})##/', $cipher, $matches)) {
+        if (preg_match('/^##(.{16})##/s', $cipher, $matches)) {
             $tag    = $matches[1];
             $cipher = substr($cipher, strlen($matches[0]));
         }
@@ -1123,7 +1123,7 @@ class rcube
      * The functions will be executed before destroying any
      * objects like smtp, imap, session, etc.
      *
-     * @param callback $function Function callback
+     * @param callable $function Function callback
      */
     public function add_shutdown_function($function)
     {
@@ -1566,7 +1566,7 @@ class rcube
     /**
      * Setter for system user object
      *
-     * @param rcube_user Current user instance
+     * @param rcube_user $user Current user instance
      */
     public function set_user($user)
     {
